# frozen_string_literal: true

module Authenticatable
  class PasswordsController < AuthenticatableController
    before_action :require_unauthenticated!

    def new
      @resource = resource_class.new
    end

    def create
      if (@resource = resource_class.find_by_identifier(identifier))
        @resource&.reset_password_token!
        Authenticatable::Mailer.reset_password(@resource, resource_name).deliver_now
      end

      set_flash_message(:notice, :create, { identifier_column: identifier_column })
      redirect_to new_password_path(resource_name)
    end

    def edit
      @resource = resource_class.find_by_reset_password_token(params[:token])
      return unless @resource.nil?

      set_flash_message(:alert, :invalid_token)
      redirect_to new_password_path(resource_name)
    end

    def update
      unless (@resource = resource_class.find_by_reset_password_token(params[:token]))
        set_flash_message(:alert, :invalid_token)
        return redirect_to new_password_path(resource_name)
      end

      return render :edit, status: :unprocessable_entity unless @resource.update_password(edit_params)

      set_flash_message(:notice, :updated)
      redirect_to new_session_path(resource_name)
    end

    private

    def edit_params
      params.require(resource_name).permit(:password, :password_confirmation)
    end

    def identifier
      params.dig(resource_name, resource_class.identifier_column.to_s)
    end
  end
end
