# frozen_string_literal: true

module Authenticatable
  class SessionsController < AuthenticatableController
    before_action :require_unauthenticated!, except: :destroy

    def new
      @resource = resource_class.new
    end

    def create
      @resource = sign_in(params, :session)
      if signed_in?
        set_flash_message(:notice, :success, { identifier: @resource.identifier })
        redirect_to root_path
      else
        set_flash_message(:alert, :invalid_credentials, { identifier: identifier_label.downcase })
        redirect_to new_session_path(resource_name)
      end
    end

    def destroy
      @resource = authenticatable.authenticate!(resource_name)
      sign_out(@resource, :session) unless @resource.nil?
      set_flash_message(:notice, :destroy)
      redirect_to new_user_session_path
    end

    private

    def signed_in?
      !!authenticatable.authenticate(resource_name)
    end

    # Authenticate and sign in a user with a provided identifier and password
    def sign_in(params, serializer)
      identifier = params.dig(resource_name, resource_class.identifier_column.to_s)
      password = params.dig(resource_name, "password")
      record = resource_class.authenticate_with_identifier(identifier, password)
      authenticatable.sign_in(record, serializer) unless record.nil?
      record || nil
    end

    # Sign out the passed resource.
    def sign_out(resource, serializer)
      authenticatable.sign_out(resource, serializer)
    end
  end
end
