# frozen_string_literal: true

require "active_support/concern"

module Authenticatable
  module Controllers
    module UrlHelpers
      extend ActiveSupport::Concern

      included do
        if respond_to?(:helper_method)
          helper_method :session_path, :new_session_path,
                        :registration_path, :new_registration_path,
                        :new_password_path, :password_path, :edit_password_url, :update_password_path
        end
      end

      # Helper methods to generate a generic path helpers dynamically
      # in views, based on the current scope.
      # Example: session_path('user') => user_session_path
      def new_session_path(resource_name)
        :"new_#{resource_name}_session"
      end

      def session_path(resource_name)
        :"#{resource_name}_session"
      end

      def new_registration_path(resource_name)
        :"new_#{resource_name}_registration"
      end

      def registration_path(resource_name)
        :"#{resource_name}_registration"
      end

      def new_password_path(resource_name)
        :"new_#{resource_name}_password"
      end

      def password_path(resource_name)
        :"#{resource_name}_password"
      end

      def update_password_path(resource_name, token)
        public_send("update_#{resource_name}_password_path", token: token)
      end

      def edit_password_url(resource_name, token)
        public_send("edit_#{resource_name}_password_url", token: token)
      end

      # End generic path helpers.

      # The default url to redirect to after sign in. This URL
      # can be overriden in your ApplicationController like this:
      #
      #   def after_sign_in_path(resource)
      #     dashboard_path
      #   end
      #
      def after_sign_in_path(_resource, _resource_name)
        root_url
      end
    end
  end
end
