# frozen_string_literal: true

module Authenticatable
  module Serializers
    class Base
      # :api: public
      attr_accessor :env, :scope, :record

      # Initialize and prepare variables.
      def initialize(env, scope)
        @env = env
        @scope = scope
        @record = nil
      end

      # Access scope settings for current scope that was set in routes.rb
      def current_scope
        Authenticatable.scopes[scope.to_sym]
      end

      # Access the class for the current scope.
      # For example if the current scope is "user":
      #   resource_class.find(:id) == User.find(:id)
      def resource_class
        current_scope.klass
      end

      # Convenience access the rack request.
      def request
        @request ||= Rack::Request.new(@env)
      end

      # Delegate #params to request#params.
      # Example:
      #   params["foo"] == request.params["foo"]
      delegate :params, to: :request
    end
  end
end
