# frozen_string_literal: true

require "rails/generators/active_record"
require "generators/authenticatable/orm_helpers"

module ActiveRecord
  module Generators
    class AuthenticatableGenerator < ActiveRecord::Generators::Base
      argument :attributes, type: :array, default: [], banner: "field:type field:type"

      include Authenticatable::Generators::OrmHelpers
      source_root File.expand_path("templates", __dir__)

      def copy_authenticatable_migration
        if (behavior == :invoke && model_exists?) || (behavior == :revoke && migration_exists?(table_name))
          migration_template "migration_existing.tt", "#{db_migrate_path}/add_authenticatable_to_#{table_name}.rb",
                             migration_version: migration_version
        else
          migration_template "migration.tt", "#{db_migrate_path}/authenticatable_create_#{table_name}.rb",
                             migration_version: migration_version
        end
      end

      def generate_model
        invoke "active_record:model", [name], migration: false unless model_exists? && behavior == :invoke
      end

      # rubocop:disable Metrics/AbcSize
      def inject_authenticatable
        content = model_contents
        class_path = (namespaced? ? class_name.to_s.split("::") : [class_name])
        indent_depth = class_path.size - 1
        content = content.split("\n").map { |line| ("  " * indent_depth) + line }.join("\n") << "\n"

        inject_into_class(model_path, class_path.last, content) if model_exists?
      end
      # rubocop:enable Metrics/AbcSize

      def add_route
        route_content = "authenticatable :#{plural_name}"
        route route_content
      end

      private

      def model_contents
        "  authenticatable\n"
      end

      # rubocop:disable Layout/IndentationWidth
      def migration_data
<<RUBY
      ## Authenticatable
      t.string :email, null: false, default: ""
      t.string :password_digest, null: false, default: ""
      t.string :reset_password_token, null: false, default: ""
      t.datetime :reset_password_sent_at
RUBY
      end
      # rubocop:enable Layout/IndentationWidth
    end
  end
end
