# frozen_string_literal: true

module Authenticatable
  module Generators
    module OrmHelpers
      private

      def model_exists?
        File.exist?(File.join(destination_root, model_path))
      end

      def migration_exists?(table_name)
        Dir.glob("#{File.join(destination_root, db_migrate_path)}/[0-9]*_*.rb")
           .grep(/\d+_add_authenticatable_to_#{table_name}.rb$/).first
      end

      def model_path
        @model_path ||= File.join("app", "models", "#{file_path}.rb")
      end

      def migration_version
        "[#{Rails::VERSION::MAJOR}.#{Rails::VERSION::MINOR}]" if rails5_and_up?
      end

      def rails5_and_up?
        Rails::VERSION::MAJOR >= 5
      end
    end
  end
end
